<?php
# It is the context of the subscription, in which a customer can manage its VPSes.
# It must correspond to a tenant created for the subscriber in the remote application system.
if (!defined('APS_DEVELOPMENT_MODE')) define('APS_DEVELOPMENT_MODE', 'on');

require_once "aps/2/runtime.php";

/**
 * Class context
 * @type("http://aps-standard.org/samples/sample-basic/context/1.0")
 * @implements("http://aps-standard.org/types/core/subscription/service/1.0")
 */
class context extends \APS\ResourceBase
{
## Strong relation (link) to the application instance

    /**
     * @link("http://aps-standard.org/samples/sample-basic/cloud/1.0")
     * @required
     */
    public $cloud;

## Weak relation (link) to collection of VPSes
    /**
     * @link("http://aps-standard.org/samples/sample-basic/vps/1.0[]")
     */
    public $vpses;

## Strong relation with the Subscription.
## This way, we allow the service to access the operation resources
## with the limits and usage defined in the subscription.

    /**
     * @link("http://aps-standard.org/types/core/subscription/1.0")
     * @required
     */
    public $subscription;

## Link to the account type makes account attributes available to the service,
## e.g., the account (subscriber) name, and all its other data.

    /**
     * @link("http://aps-standard.org/types/core/account/1.0")
     * @required
     */
    public $account;

## Subset of attributes marked as read-only,
## which means only the application can change them.
## The password is "encrypted", which prevents customers to get its value.

    /**
     * @type(string)
     * @title("Context ID")
     * @readonly
     */

    public $contextid;

    /**
     * @type(string)
     * @title("Context Password")
     * @readonly
     * @encrypted
     */

    public $contextpass;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Total disk space Usage, GB")
     * @unit("gb")
     */
    public $diskusagetotal;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Total CPU usage, CPUcore-hour")
     * @unit("item-h")
     */
    public $cpuusagetotal;

    /**
     * @type("http://aps-standard.org/types/core/resource/1.0#Counter")
     * @description("Total memory usage, MB-hour")
     * @unit("mb-h")
     */
    public $memoryusagetotal;

    /**
     * @type("integer")
     * @description("Time stamp of the last resource usage counting")
     */
    public $time1;

    public function provision()
    {
        $this->time1 = time();
        $this->cpuusagetotal->usage = 0;
        $this->memoryusagetotal->usage = 0;
        $this->diskusagetotal->usage = 0;
    }

    public function retrieve()
    {
        $currenttime = time();
        $timedelta = ($currenttime - $this->time1) / 3600;    // Elapsed time in hours
        $this->time1 = $currenttime;

        $diskusage = 0;
        $cpuusage = 0;
        $ramusage = 0;
        foreach ($this->vpses as $vps) {
            if ($vps->state == 'Running') {
                $ramusage += $vps->hardware->memory;
                $cpuusage += $vps->hardware->CPU->number;
            }
            $diskusage += $vps->hardware->diskspace;
        }
        $this->memoryusagetotal->usage += floor($ramusage * $timedelta);    // Additive, rounded down
        $this->cpuusagetotal->usage += floor($cpuusage * $timedelta);        // Additive, rounded down
        $this->diskusagetotal->usage = $diskusage;    // Non-additive
    }

    /**
     * @verb(GET)
     * @path("/resourceCounting")
     */
    public function resourceCounting()
    {
        $this->retrieve();
        $apsc = \APS\Request::getController();
        $apsc->updateResource($this);
    }
}

